@@ -12,6 +12,29 @@ import java.util.*;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+/*
+ * jmx4perl - WAR Agent for exporting JMX via JSON
+ *
+ * Copyright (C) 2009 Roland Huß, roland@cpan.org
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * A commercial license is available as well. Please contact roland@cpan.org for
+ * further details.
+ */
+
 /**
  * Factory for creating {@link org.jmx4perl.JmxRequest}s
  *
@@ -55,7 +78,7 @@ final public class JmxRequestFactory {
      *   <li>Type: <b>write</b> ({@link Type#WRITE}<br/>
      *       Parameters: <code>param1</code> = MBean name, <code>param2</code> = Attribute name,
      *       <code>param3</code> = value, <code>param4 ... paramN</code> = Inner Path.
-     *       The value must be URL encoded (with UTF-8 as charset), and must be convertable into
+     *       The value must be URL encoded (with UTF-8 as charset), and must be convertible into
      *       a data structure</li>
      *   <li>Type: <b>exec</b> ({@link Type#EXEC}<br/>
      *       Parameters: <code>param1</code> = MBean name, <code>param2</code> = operation name,
@@ -74,10 +97,22 @@ final public class JmxRequestFactory {
     static public JmxRequest createRequestFromUrl(String pPathInfo, Map<String,String[]> pParameterMap) {
         JmxRequest request = null;
         try {
-            if (pPathInfo != null && pPathInfo.length() > 0) {
+            String pathInfo = pPathInfo;
 
+            // If no pathinfo is given directly, we look for a query parameter named 'p'.
+            // This variant is helpful, if there are problems with the server mangling
+            // up the pathinfo (e.g. for security concerns, often '/','\',';' and other are not
+            // allowed in encoded form within the pathinfo)
+            if (pPathInfo == null || pPathInfo.length() == 0 || pathInfo.matches("^/+$")) {
+                String[] vals = pParameterMap.get("p");
+                if (vals != null && vals.length > 0) {
+                    pathInfo = vals[0];
+                }
+            }
+
+            if (pathInfo != null && pathInfo.length() > 0) {
                 // Get all path elements as a reverse stack
-                Stack<String> elements = extractElementsFromPath(pPathInfo);
+                Stack<String> elements = extractElementsFromPath(pathInfo);
                 Type type = extractType(elements.pop());
 
                 Processor processor = PROCESSOR_MAP.get(type);
@@ -89,12 +124,14 @@ final public class JmxRequestFactory {
                 request = processor.process(elements);
 
                 // Extract all additional args from the remaining path info
-                request.setExtraArgs(toList(elements));
+                request.setExtraArgs(prepareExtraArgs(elements));
 
                 // Setup JSON representation
                 extractParameters(request,pParameterMap);
+                return request;
+            } else {
+                throw new IllegalArgumentException("No pathinfo given and no query parameter 'p'");
             }
-            return request;
         } catch (NoSuchElementException exp) {
             throw new IllegalArgumentException("Invalid path info " + pPathInfo,exp);
         } catch (MalformedObjectNameException e) {
@@ -182,7 +219,7 @@ final public class JmxRequestFactory {
             }
             StringBuffer val;
 
-            // Special escape at the beginning indicates that the this element belongs
+            // Special escape at the beginning indicates that this element belongs
             // to the next one
             if (element.substring(0,1).equals("^")) {
                 val = new StringBuffer();
@@ -191,7 +228,7 @@ final public class JmxRequestFactory {
             } else {
                 val = previousBuffer;
             }
-            // Append approp. nr of slashes
+            // Append appropriate nr of slashes
             for (int j=0;j<element.length();j++) {
                 val.append("/");
             }
@@ -230,12 +267,17 @@ final public class JmxRequestFactory {
         throw new IllegalArgumentException("Invalid request type '" + pTypeS + "'");
     }
 
-    private static List<String>toList(Stack<String> pElements) {
-        List<String> p = new ArrayList<String>();
+    private static List<String> prepareExtraArgs(Stack<String> pElements) {
+        List<String> ret = new ArrayList<String>();
         while (!pElements.isEmpty()) {
-            p.add(pElements.pop());
+            String element = pElements.pop();
+            // Check for escapes
+            while (element.endsWith("\\") && !pElements.isEmpty()) {
+                element = element.substring(0,element.length() - 1) + "/" + pElements.pop();
+            }
+            ret.add(element);
         }
-        return p;
+        return ret;
     }
 
     private static void extractParameters(JmxRequest pRequest,Map<String,String[]> pParameterMap) {
@@ -288,7 +330,6 @@ final public class JmxRequestFactory {
                 return req;
             }
         });
-
         PROCESSOR_MAP.put(Type.LIST,new Processor() {
             public JmxRequest process(Stack<String> e) throws MalformedObjectNameException {
                 return new JmxRequest(Type.LIST);
